=begin

gridmap.rb - keeps track of nodes on the grid, mapping [x,y] to switch. 
Functions found here: 

Switch
    ::has_node?(n)  - checks if node n is on this switch
    ::get_port(n)   - looks up which port node n is connected to 
    ::get_ip()      - return doted decimal IP of switch

    'id' is last octet of switch IP. 

Grid
    ::get_node_id(row, col) - find node ID of node at [row,col]
    ::switch_of(x,<y>) - find switch that node x or [x,y] is conneced to,   
                       returns [switch ID, switchport] pair.

=end


#port number a range of 48 values between 1 and 400, to match grid.         
#ID is last octet of IP address 172.16.20.*         
class Switch
    attr_accessor :upper, :lower, :pmap, :id
    NUM_PORTS = 48
    BASE_ADDR = "172.16.20."

    #Take into account that even ports on top, odd on bottom. 48 Nodes
    #are wired in zigzag fashion, e.g. top ports <-> first 24, bottom 
    #to last 24.               
    def initialize(min, max, id)
        unless (max - min + 1) > NUM_PORTS
            @upper = (min..(min + NUM_PORTS/2 - 1))
            @lower = ((min + NUM_PORTS/2)..max)
            @pmap = @lower.entries.zip(@upper.entries).flatten
        else
            raise "Switch::initialize: tried adding too many ports"
        end
        @id = id
    end

    #check if node ID id is on switch
    def has_node?(n)
        return @pmap.include?(n)
    end

    def get_ip()
        return BASE_ADDR + (id.to_s)
    end

    #return true port number (1..48) associated with an ID.  
    def get_port(n)
        p = @pmap.index(n)

        unless p.nil? then return p + 1 else raise "Switch::get_port: node not found" end
    end
end

#map [x,y] to (1..400) and a switch. Hard-coded because I am bad.  
class Grid
    Rows = 20
    Cols = 20

    SW_LIST = [ Switch.new(1, 48, 1),
                Switch.new(49, 96, 2),
                Switch.new(97, 144, 3),
                Switch.new(145, 192, 4),
                Switch.new(193, 240, 5),
                Switch.new(241, 288, 6),
                Switch.new(289, 336, 7),
                Switch.new(337, 384, 8),
                Switch.new(385, 432, 253) ]

    #node number of first row of grid. 
    FIRST = [400, 380, 360, 340, 320, 300, 280, 260, 240, 220, 200, 180, 160, 140, 120, 100, 80, 60, 40, 20]

    def initialize()
    end

    #(row - 1) is offset of the ID from the first node of the col-th column.
    def get_node_id(row, col)
        return FIRST[col - 1] - (row - 1)
    end

    #get [switch id, switchport] pair for a node 
    def switch_of(row, col=nil)
        sw_port = nil
        sw_id = nil
        node = nil

        if col.nil? then node = row else node = get_node_id(row, col) end

        SW_LIST.each do |sw|
            if sw.has_node?(node)
                sw_port = sw.get_port(node)
                sw_id = sw.id
                break
            end
        end

        if (sw_port.nil?) or (sw_id.nil?)
            return nil
        end

        return [sw_id, sw_port]
    end
end


